-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
-- P8. Tutorial on the structure of a CLK generator:
-- Example application to be used in the LAB5 demonstration board.
-- https://digsys.upc.edu/csd/P05/lab05/lab5.html#prototyping

-- The MAX10-Lite board crystal oscillator chip is OSC_CLK_in = 50 MHz
-- CLK_A_SQ, this is a square waveform of 200 Hz 
-- CLK_B_SQ, this is a square waveform of 10 Hz 
-- CLK_C_SQ, this is a square waveform of 1 Hz 

-- Plan: Frequency dividers and T_FF to square the pulsed terminal count signals    
-- Count Enable (CE) and asynchronous active-high clear direct (CD)
-- 3 cascaded frequency dividers in series chaining TC --> CE 
-- 3 T_FF to square the pulsed waveforms (while dividing by 2)
-- The reference schematic is presented in lecture: 
-- https://digsys.upc.edu/csd/P08/L08_2/L8_2.html 
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	CD,CE			: 	IN  std_logic;
			OSC_CLK_in		: 	IN  std_logic;
			CLK_A_SQ		:	OUT std_logic;
			CLK_B_SQ		:	OUT std_logic;
			CLK_C_SQ		:	OUT std_logic			
          );
END CLK_Generator ;

ARCHITECTURE schematic OF CLK_Generator IS  
-- Components
	COMPONENT Freq_div_125000 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC125000	:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_div_20 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC20		:	OUT std_logic
		);
	END COMPONENT;

	COMPONENT Freq_div_10 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC10		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT T_FF IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK			:	std_logic;
	SIGNAL Pulsed_2A	:	std_logic;
	SIGNAL Pulsed_2B	:	std_logic;
	SIGNAL Pulsed_2C	:	std_logic;	
	
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_125000
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> CE,
			TC125000	=> Pulsed_2A					  
		);

	Chip2	:  Freq_Div_20
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_2A,
			TC20 	=> Pulsed_2B					  
		);

	Chip3	:  Freq_Div_10
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_2B,
			TC10 	=> Pulsed_2C					  
		);

	Chip4	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2A,
			Q 		=> CLK_A_SQ				  
		);		

	Chip5	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2B,
			Q 		=> CLK_B_SQ					  
		);		

	Chip6	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2C,
			Q 		=> CLK_C_SQ					  
		);		
			
			
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;
	
END schematic ;
